@@ -10,7 +10,7 @@ class RegressionModel(Model):
 
 class XGBoostRegressionModel():
     """    
-    subclass of ml_petrophysics.model
+    subclass of akerbp.model
 
     Attributes
     ----------
