@@ -1,18 +1,20 @@
 import numpy as np
 from akerbp.models.rule_based_models import helpers
 from akerbp.models.rule_based_models import BS_helpers
+from pyod.utils.utility import standardizer #returns z-score
 
-def flag_washout(df_well, y_pred=None, **kwargs):
+def get_outliers(df_well, y_pred=None):
+    
     """
-    Returns anomalous CALI-BS
-
+    Returns CALI-BS outliers
+    
     Args:
-        df_well (pd.DataFrame): [description]
-
+        df_well (pd.DataFrame): well data
+        y_pred (pd.DataFrame): dataframe with flags
+    
     Returns:
-        [type]: [description]
-    """
-    print('Method: washout...')
+        pd.DataFrame: y_pred with added flag columns
+    """ 
     if 'CALI' in df_well.columns and 'BS' in df_well.columns:
         df_well.loc[:, 'CALI_BS'] = np.abs(df_well['CALI'] - df_well['BS'])
 
@@ -55,4 +57,30 @@ def flag_washout(df_well, y_pred=None, **kwargs):
                 )
             )
     y_pred.loc[den_washout_anomalies, ['flag_washout_gen', 'flag_washout_den']] = 1
+
+    return y_pred
+        
+def flag_washout(df_well, y_pred=None, **kwargs):
+    """
+    Returns anomalous CALI-BS
+
+    Args:
+        df_well (pd.DataFrame): well data 
+        y_pred (pd.DataFrame): dataframe to add detection columns to
+
+    Returns:
+        pd.DataFrame: y_pred with the added columns
+    """
+    print('Method: washout...')
+
+    y_pred = get_outliers(df_well, y_pred)
+    rolling_window = kwargs.get('washout_rolling_window', 101)
+    for bsr in y_pred['BS_region'].unique():
+        df_gr = y_pred[y_pred['BS_region']==bsr].copy()
+        rows_with_values = df_gr[df_gr['CALI'].isna()==False].index
+        if len(rows_with_values):
+            df_gr.loc[rows_with_values, 'CALI_std'] = df_gr.loc[rows_with_values, 'CALI'].rolling(rolling_window).std().fillna(0)
+            y_pred.loc[rows_with_values, 'score_washout_gen'] = np.abs(standardizer(df_gr.loc[rows_with_values, 'CALI_std'].values.reshape(-1,1)))
+
+    y_pred['score_washout_gen'].fillna(0, inplace=True)
     return y_pred
\ No newline at end of file
